# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='messages.proto',
  package='pomagma.protobuf',
  serialized_pb='\n\x0emessages.proto\x12\x10pomagma.protobuf\"\xf0\x08\n\x13\x43\x61rtographerRequest\x12\x38\n\x04\x63rop\x18\n \x01(\x0b\x32*.pomagma.protobuf.CartographerRequest.Crop\x12>\n\x07\x64\x65\x63lare\x18\x0b \x01(\x0b\x32-.pomagma.protobuf.CartographerRequest.Declare\x12<\n\x06\x61ssume\x18\x0c \x01(\x0b\x32,.pomagma.protobuf.CartographerRequest.Assume\x12:\n\x05infer\x18\r \x01(\x0b\x32+.pomagma.protobuf.CartographerRequest.Infer\x12>\n\x07\x65xecute\x18\x0e \x01(\x0b\x32-.pomagma.protobuf.CartographerRequest.Execute\x12\x42\n\taggregate\x18\x14 \x01(\x0b\x32/.pomagma.protobuf.CartographerRequest.Aggregate\x12@\n\x08validate\x18\x1e \x01(\x0b\x32..pomagma.protobuf.CartographerRequest.Validate\x12\x38\n\x04info\x18( \x01(\x0b\x32*.pomagma.protobuf.CartographerRequest.Info\x12\x38\n\x04\x64ump\x18) \x01(\x0b\x32*.pomagma.protobuf.CartographerRequest.Dump\x12\x38\n\x04trim\x18* \x03(\x0b\x32*.pomagma.protobuf.CartographerRequest.Trim\x12\x44\n\nconjecture\x18+ \x01(\x0b\x32\x30.pomagma.protobuf.CartographerRequest.Conjecture\x12\x38\n\x04stop\x18\x63 \x01(\x0b\x32*.pomagma.protobuf.CartographerRequest.Stop\x1a\x1b\n\x04\x43rop\x12\x13\n\x08headroom\x18\x01 \x01(\r:\x01\x30\x1a$\n\x07\x44\x65\x63lare\x12\x19\n\x11nullary_functions\x18\x01 \x03(\t\x1a\x1a\n\x06\x41ssume\x12\x10\n\x08\x66\x61\x63ts_in\x18\x01 \x02(\t\x1a\x19\n\x05Infer\x12\x10\n\x08priority\x18\x01 \x02(\r\x1a\x1a\n\x07\x45xecute\x12\x0f\n\x07program\x18\x01 \x02(\t\x1a\x1e\n\tAggregate\x12\x11\n\tsurvey_in\x18\x01 \x02(\t\x1a\n\n\x08Validate\x1a\x06\n\x04Info\x1a\x19\n\x04\x44ump\x12\x11\n\tworld_out\x18\x01 \x02(\t\x1a;\n\x04Trim\x12\x0c\n\x04size\x18\x01 \x02(\r\x12\x13\n\x0btemperature\x18\x02 \x02(\x08\x12\x10\n\x08\x66ilename\x18\x03 \x02(\t\x1aG\n\nConjecture\x12\x13\n\x0b\x64iverge_out\x18\x01 \x02(\t\x12\x11\n\tequal_out\x18\x02 \x02(\t\x12\x11\n\tmax_count\x18\x03 \x02(\x04\x1a\x06\n\x04Stop\"\xab\x08\n\x14\x43\x61rtographerResponse\x12\x39\n\x04\x63rop\x18\n \x01(\x0b\x32+.pomagma.protobuf.CartographerResponse.Crop\x12?\n\x07\x64\x65\x63lare\x18\x0b \x01(\x0b\x32..pomagma.protobuf.CartographerResponse.Declare\x12=\n\x06\x61ssume\x18\x0c \x01(\x0b\x32-.pomagma.protobuf.CartographerResponse.Assume\x12;\n\x05infer\x18\r \x01(\x0b\x32,.pomagma.protobuf.CartographerResponse.Infer\x12?\n\x07\x65xecute\x18\x0e \x01(\x0b\x32..pomagma.protobuf.CartographerResponse.Execute\x12\x43\n\taggregate\x18\x14 \x01(\x0b\x32\x30.pomagma.protobuf.CartographerResponse.Aggregate\x12\x41\n\x08validate\x18\x1e \x01(\x0b\x32/.pomagma.protobuf.CartographerResponse.Validate\x12\x39\n\x04info\x18( \x01(\x0b\x32+.pomagma.protobuf.CartographerResponse.Info\x12\x39\n\x04\x64ump\x18) \x01(\x0b\x32+.pomagma.protobuf.CartographerResponse.Dump\x12\x39\n\x04trim\x18* \x03(\x0b\x32+.pomagma.protobuf.CartographerResponse.Trim\x12\x45\n\nconjecture\x18+ \x01(\x0b\x32\x31.pomagma.protobuf.CartographerResponse.Conjecture\x12\x39\n\x04stop\x18\x63 \x01(\x0b\x32+.pomagma.protobuf.CartographerResponse.Stop\x1a\x06\n\x04\x43rop\x1a\t\n\x07\x44\x65\x63lare\x1aZ\n\x06\x41ssume\x12\x11\n\tpos_count\x18\x01 \x02(\x04\x12\x11\n\tneg_count\x18\x02 \x02(\x04\x12\x13\n\x0bmerge_count\x18\x03 \x02(\x04\x12\x15\n\rignored_count\x18\x04 \x02(\x04\x1a\x1e\n\x05Infer\x12\x15\n\rtheorem_count\x18\x01 \x02(\x04\x1a\t\n\x07\x45xecute\x1a\x0b\n\tAggregate\x1a\n\n\x08Validate\x1a\x1a\n\x04Info\x12\x12\n\nitem_count\x18\x01 \x02(\r\x1a\x06\n\x04\x44ump\x1a\x06\n\x04Trim\x1a\x38\n\nConjecture\x12\x15\n\rdiverge_count\x18\x01 \x02(\x04\x12\x13\n\x0b\x65qual_count\x18\x02 \x02(\x04\x1a\x06\n\x04Stop')




_CARTOGRAPHERREQUEST_CROP = _descriptor.Descriptor(
  name='Crop',
  full_name='pomagma.protobuf.CartographerRequest.Crop',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='headroom', full_name='pomagma.protobuf.CartographerRequest.Crop.headroom', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=804,
  serialized_end=831,
)

_CARTOGRAPHERREQUEST_DECLARE = _descriptor.Descriptor(
  name='Declare',
  full_name='pomagma.protobuf.CartographerRequest.Declare',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nullary_functions', full_name='pomagma.protobuf.CartographerRequest.Declare.nullary_functions', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=833,
  serialized_end=869,
)

_CARTOGRAPHERREQUEST_ASSUME = _descriptor.Descriptor(
  name='Assume',
  full_name='pomagma.protobuf.CartographerRequest.Assume',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='facts_in', full_name='pomagma.protobuf.CartographerRequest.Assume.facts_in', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=871,
  serialized_end=897,
)

_CARTOGRAPHERREQUEST_INFER = _descriptor.Descriptor(
  name='Infer',
  full_name='pomagma.protobuf.CartographerRequest.Infer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='priority', full_name='pomagma.protobuf.CartographerRequest.Infer.priority', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=899,
  serialized_end=924,
)

_CARTOGRAPHERREQUEST_EXECUTE = _descriptor.Descriptor(
  name='Execute',
  full_name='pomagma.protobuf.CartographerRequest.Execute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='program', full_name='pomagma.protobuf.CartographerRequest.Execute.program', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=926,
  serialized_end=952,
)

_CARTOGRAPHERREQUEST_AGGREGATE = _descriptor.Descriptor(
  name='Aggregate',
  full_name='pomagma.protobuf.CartographerRequest.Aggregate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='survey_in', full_name='pomagma.protobuf.CartographerRequest.Aggregate.survey_in', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=954,
  serialized_end=984,
)

_CARTOGRAPHERREQUEST_VALIDATE = _descriptor.Descriptor(
  name='Validate',
  full_name='pomagma.protobuf.CartographerRequest.Validate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=986,
  serialized_end=996,
)

_CARTOGRAPHERREQUEST_INFO = _descriptor.Descriptor(
  name='Info',
  full_name='pomagma.protobuf.CartographerRequest.Info',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=998,
  serialized_end=1004,
)

_CARTOGRAPHERREQUEST_DUMP = _descriptor.Descriptor(
  name='Dump',
  full_name='pomagma.protobuf.CartographerRequest.Dump',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='world_out', full_name='pomagma.protobuf.CartographerRequest.Dump.world_out', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1006,
  serialized_end=1031,
)

_CARTOGRAPHERREQUEST_TRIM = _descriptor.Descriptor(
  name='Trim',
  full_name='pomagma.protobuf.CartographerRequest.Trim',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='pomagma.protobuf.CartographerRequest.Trim.size', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='temperature', full_name='pomagma.protobuf.CartographerRequest.Trim.temperature', index=1,
      number=2, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filename', full_name='pomagma.protobuf.CartographerRequest.Trim.filename', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1033,
  serialized_end=1092,
)

_CARTOGRAPHERREQUEST_CONJECTURE = _descriptor.Descriptor(
  name='Conjecture',
  full_name='pomagma.protobuf.CartographerRequest.Conjecture',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='diverge_out', full_name='pomagma.protobuf.CartographerRequest.Conjecture.diverge_out', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='equal_out', full_name='pomagma.protobuf.CartographerRequest.Conjecture.equal_out', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_count', full_name='pomagma.protobuf.CartographerRequest.Conjecture.max_count', index=2,
      number=3, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1094,
  serialized_end=1165,
)

_CARTOGRAPHERREQUEST_STOP = _descriptor.Descriptor(
  name='Stop',
  full_name='pomagma.protobuf.CartographerRequest.Stop',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1167,
  serialized_end=1173,
)

_CARTOGRAPHERREQUEST = _descriptor.Descriptor(
  name='CartographerRequest',
  full_name='pomagma.protobuf.CartographerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='crop', full_name='pomagma.protobuf.CartographerRequest.crop', index=0,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='declare', full_name='pomagma.protobuf.CartographerRequest.declare', index=1,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='assume', full_name='pomagma.protobuf.CartographerRequest.assume', index=2,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='infer', full_name='pomagma.protobuf.CartographerRequest.infer', index=3,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='execute', full_name='pomagma.protobuf.CartographerRequest.execute', index=4,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='aggregate', full_name='pomagma.protobuf.CartographerRequest.aggregate', index=5,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='validate', full_name='pomagma.protobuf.CartographerRequest.validate', index=6,
      number=30, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info', full_name='pomagma.protobuf.CartographerRequest.info', index=7,
      number=40, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dump', full_name='pomagma.protobuf.CartographerRequest.dump', index=8,
      number=41, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trim', full_name='pomagma.protobuf.CartographerRequest.trim', index=9,
      number=42, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='conjecture', full_name='pomagma.protobuf.CartographerRequest.conjecture', index=10,
      number=43, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stop', full_name='pomagma.protobuf.CartographerRequest.stop', index=11,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CARTOGRAPHERREQUEST_CROP, _CARTOGRAPHERREQUEST_DECLARE, _CARTOGRAPHERREQUEST_ASSUME, _CARTOGRAPHERREQUEST_INFER, _CARTOGRAPHERREQUEST_EXECUTE, _CARTOGRAPHERREQUEST_AGGREGATE, _CARTOGRAPHERREQUEST_VALIDATE, _CARTOGRAPHERREQUEST_INFO, _CARTOGRAPHERREQUEST_DUMP, _CARTOGRAPHERREQUEST_TRIM, _CARTOGRAPHERREQUEST_CONJECTURE, _CARTOGRAPHERREQUEST_STOP, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=37,
  serialized_end=1173,
)


_CARTOGRAPHERRESPONSE_CROP = _descriptor.Descriptor(
  name='Crop',
  full_name='pomagma.protobuf.CartographerResponse.Crop',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=804,
  serialized_end=810,
)

_CARTOGRAPHERRESPONSE_DECLARE = _descriptor.Descriptor(
  name='Declare',
  full_name='pomagma.protobuf.CartographerResponse.Declare',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=833,
  serialized_end=842,
)

_CARTOGRAPHERRESPONSE_ASSUME = _descriptor.Descriptor(
  name='Assume',
  full_name='pomagma.protobuf.CartographerResponse.Assume',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pos_count', full_name='pomagma.protobuf.CartographerResponse.Assume.pos_count', index=0,
      number=1, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='neg_count', full_name='pomagma.protobuf.CartographerResponse.Assume.neg_count', index=1,
      number=2, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='merge_count', full_name='pomagma.protobuf.CartographerResponse.Assume.merge_count', index=2,
      number=3, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ignored_count', full_name='pomagma.protobuf.CartographerResponse.Assume.ignored_count', index=3,
      number=4, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1975,
  serialized_end=2065,
)

_CARTOGRAPHERRESPONSE_INFER = _descriptor.Descriptor(
  name='Infer',
  full_name='pomagma.protobuf.CartographerResponse.Infer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='theorem_count', full_name='pomagma.protobuf.CartographerResponse.Infer.theorem_count', index=0,
      number=1, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2067,
  serialized_end=2097,
)

_CARTOGRAPHERRESPONSE_EXECUTE = _descriptor.Descriptor(
  name='Execute',
  full_name='pomagma.protobuf.CartographerResponse.Execute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=926,
  serialized_end=935,
)

_CARTOGRAPHERRESPONSE_AGGREGATE = _descriptor.Descriptor(
  name='Aggregate',
  full_name='pomagma.protobuf.CartographerResponse.Aggregate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=954,
  serialized_end=965,
)

_CARTOGRAPHERRESPONSE_VALIDATE = _descriptor.Descriptor(
  name='Validate',
  full_name='pomagma.protobuf.CartographerResponse.Validate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=986,
  serialized_end=996,
)

_CARTOGRAPHERRESPONSE_INFO = _descriptor.Descriptor(
  name='Info',
  full_name='pomagma.protobuf.CartographerResponse.Info',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='item_count', full_name='pomagma.protobuf.CartographerResponse.Info.item_count', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2135,
  serialized_end=2161,
)

_CARTOGRAPHERRESPONSE_DUMP = _descriptor.Descriptor(
  name='Dump',
  full_name='pomagma.protobuf.CartographerResponse.Dump',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1006,
  serialized_end=1012,
)

_CARTOGRAPHERRESPONSE_TRIM = _descriptor.Descriptor(
  name='Trim',
  full_name='pomagma.protobuf.CartographerResponse.Trim',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1033,
  serialized_end=1039,
)

_CARTOGRAPHERRESPONSE_CONJECTURE = _descriptor.Descriptor(
  name='Conjecture',
  full_name='pomagma.protobuf.CartographerResponse.Conjecture',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='diverge_count', full_name='pomagma.protobuf.CartographerResponse.Conjecture.diverge_count', index=0,
      number=1, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='equal_count', full_name='pomagma.protobuf.CartographerResponse.Conjecture.equal_count', index=1,
      number=2, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2179,
  serialized_end=2235,
)

_CARTOGRAPHERRESPONSE_STOP = _descriptor.Descriptor(
  name='Stop',
  full_name='pomagma.protobuf.CartographerResponse.Stop',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1167,
  serialized_end=1173,
)

_CARTOGRAPHERRESPONSE = _descriptor.Descriptor(
  name='CartographerResponse',
  full_name='pomagma.protobuf.CartographerResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='crop', full_name='pomagma.protobuf.CartographerResponse.crop', index=0,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='declare', full_name='pomagma.protobuf.CartographerResponse.declare', index=1,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='assume', full_name='pomagma.protobuf.CartographerResponse.assume', index=2,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='infer', full_name='pomagma.protobuf.CartographerResponse.infer', index=3,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='execute', full_name='pomagma.protobuf.CartographerResponse.execute', index=4,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='aggregate', full_name='pomagma.protobuf.CartographerResponse.aggregate', index=5,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='validate', full_name='pomagma.protobuf.CartographerResponse.validate', index=6,
      number=30, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info', full_name='pomagma.protobuf.CartographerResponse.info', index=7,
      number=40, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dump', full_name='pomagma.protobuf.CartographerResponse.dump', index=8,
      number=41, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trim', full_name='pomagma.protobuf.CartographerResponse.trim', index=9,
      number=42, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='conjecture', full_name='pomagma.protobuf.CartographerResponse.conjecture', index=10,
      number=43, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stop', full_name='pomagma.protobuf.CartographerResponse.stop', index=11,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CARTOGRAPHERRESPONSE_CROP, _CARTOGRAPHERRESPONSE_DECLARE, _CARTOGRAPHERRESPONSE_ASSUME, _CARTOGRAPHERRESPONSE_INFER, _CARTOGRAPHERRESPONSE_EXECUTE, _CARTOGRAPHERRESPONSE_AGGREGATE, _CARTOGRAPHERRESPONSE_VALIDATE, _CARTOGRAPHERRESPONSE_INFO, _CARTOGRAPHERRESPONSE_DUMP, _CARTOGRAPHERRESPONSE_TRIM, _CARTOGRAPHERRESPONSE_CONJECTURE, _CARTOGRAPHERRESPONSE_STOP, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1176,
  serialized_end=2243,
)

_CARTOGRAPHERREQUEST_CROP.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_DECLARE.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_ASSUME.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_INFER.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_EXECUTE.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_AGGREGATE.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_VALIDATE.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_INFO.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_DUMP.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_TRIM.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_CONJECTURE.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST_STOP.containing_type = _CARTOGRAPHERREQUEST;
_CARTOGRAPHERREQUEST.fields_by_name['crop'].message_type = _CARTOGRAPHERREQUEST_CROP
_CARTOGRAPHERREQUEST.fields_by_name['declare'].message_type = _CARTOGRAPHERREQUEST_DECLARE
_CARTOGRAPHERREQUEST.fields_by_name['assume'].message_type = _CARTOGRAPHERREQUEST_ASSUME
_CARTOGRAPHERREQUEST.fields_by_name['infer'].message_type = _CARTOGRAPHERREQUEST_INFER
_CARTOGRAPHERREQUEST.fields_by_name['execute'].message_type = _CARTOGRAPHERREQUEST_EXECUTE
_CARTOGRAPHERREQUEST.fields_by_name['aggregate'].message_type = _CARTOGRAPHERREQUEST_AGGREGATE
_CARTOGRAPHERREQUEST.fields_by_name['validate'].message_type = _CARTOGRAPHERREQUEST_VALIDATE
_CARTOGRAPHERREQUEST.fields_by_name['info'].message_type = _CARTOGRAPHERREQUEST_INFO
_CARTOGRAPHERREQUEST.fields_by_name['dump'].message_type = _CARTOGRAPHERREQUEST_DUMP
_CARTOGRAPHERREQUEST.fields_by_name['trim'].message_type = _CARTOGRAPHERREQUEST_TRIM
_CARTOGRAPHERREQUEST.fields_by_name['conjecture'].message_type = _CARTOGRAPHERREQUEST_CONJECTURE
_CARTOGRAPHERREQUEST.fields_by_name['stop'].message_type = _CARTOGRAPHERREQUEST_STOP
_CARTOGRAPHERRESPONSE_CROP.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_DECLARE.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_ASSUME.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_INFER.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_EXECUTE.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_AGGREGATE.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_VALIDATE.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_INFO.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_DUMP.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_TRIM.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_CONJECTURE.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE_STOP.containing_type = _CARTOGRAPHERRESPONSE;
_CARTOGRAPHERRESPONSE.fields_by_name['crop'].message_type = _CARTOGRAPHERRESPONSE_CROP
_CARTOGRAPHERRESPONSE.fields_by_name['declare'].message_type = _CARTOGRAPHERRESPONSE_DECLARE
_CARTOGRAPHERRESPONSE.fields_by_name['assume'].message_type = _CARTOGRAPHERRESPONSE_ASSUME
_CARTOGRAPHERRESPONSE.fields_by_name['infer'].message_type = _CARTOGRAPHERRESPONSE_INFER
_CARTOGRAPHERRESPONSE.fields_by_name['execute'].message_type = _CARTOGRAPHERRESPONSE_EXECUTE
_CARTOGRAPHERRESPONSE.fields_by_name['aggregate'].message_type = _CARTOGRAPHERRESPONSE_AGGREGATE
_CARTOGRAPHERRESPONSE.fields_by_name['validate'].message_type = _CARTOGRAPHERRESPONSE_VALIDATE
_CARTOGRAPHERRESPONSE.fields_by_name['info'].message_type = _CARTOGRAPHERRESPONSE_INFO
_CARTOGRAPHERRESPONSE.fields_by_name['dump'].message_type = _CARTOGRAPHERRESPONSE_DUMP
_CARTOGRAPHERRESPONSE.fields_by_name['trim'].message_type = _CARTOGRAPHERRESPONSE_TRIM
_CARTOGRAPHERRESPONSE.fields_by_name['conjecture'].message_type = _CARTOGRAPHERRESPONSE_CONJECTURE
_CARTOGRAPHERRESPONSE.fields_by_name['stop'].message_type = _CARTOGRAPHERRESPONSE_STOP
DESCRIPTOR.message_types_by_name['CartographerRequest'] = _CARTOGRAPHERREQUEST
DESCRIPTOR.message_types_by_name['CartographerResponse'] = _CARTOGRAPHERRESPONSE

class CartographerRequest(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Crop(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_CROP

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Crop)

  class Declare(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_DECLARE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Declare)

  class Assume(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_ASSUME

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Assume)

  class Infer(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_INFER

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Infer)

  class Execute(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_EXECUTE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Execute)

  class Aggregate(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_AGGREGATE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Aggregate)

  class Validate(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_VALIDATE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Validate)

  class Info(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_INFO

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Info)

  class Dump(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_DUMP

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Dump)

  class Trim(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_TRIM

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Trim)

  class Conjecture(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_CONJECTURE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Conjecture)

  class Stop(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERREQUEST_STOP

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest.Stop)
  DESCRIPTOR = _CARTOGRAPHERREQUEST

  # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerRequest)

class CartographerResponse(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Crop(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_CROP

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Crop)

  class Declare(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_DECLARE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Declare)

  class Assume(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_ASSUME

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Assume)

  class Infer(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_INFER

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Infer)

  class Execute(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_EXECUTE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Execute)

  class Aggregate(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_AGGREGATE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Aggregate)

  class Validate(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_VALIDATE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Validate)

  class Info(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_INFO

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Info)

  class Dump(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_DUMP

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Dump)

  class Trim(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_TRIM

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Trim)

  class Conjecture(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_CONJECTURE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Conjecture)

  class Stop(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CARTOGRAPHERRESPONSE_STOP

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse.Stop)
  DESCRIPTOR = _CARTOGRAPHERRESPONSE

  # @@protoc_insertion_point(class_scope:pomagma.protobuf.CartographerResponse)


# @@protoc_insertion_point(module_scope)
