# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protobuf_test.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protobuf_test.proto',
  package='pomagma.protobuf',
  serialized_pb='\n\x13protobuf_test.proto\x12\x10pomagma.protobuf\"\xb3\x01\n\x0bTestMessage\x12\x17\n\x0foptional_string\x18\x01 \x01(\t\x12\x17\n\x0frepeated_string\x18\x16 \x03(\t\x12\x38\n\x10optional_message\x18\xcd\x02 \x01(\x0b\x32\x1d.pomagma.protobuf.TestMessage\x12\x38\n\x10repeated_message\x18\xdc\" \x03(\x0b\x32\x1d.pomagma.protobuf.TestMessage')




_TESTMESSAGE = _descriptor.Descriptor(
  name='TestMessage',
  full_name='pomagma.protobuf.TestMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='optional_string', full_name='pomagma.protobuf.TestMessage.optional_string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='repeated_string', full_name='pomagma.protobuf.TestMessage.repeated_string', index=1,
      number=22, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='optional_message', full_name='pomagma.protobuf.TestMessage.optional_message', index=2,
      number=333, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='repeated_message', full_name='pomagma.protobuf.TestMessage.repeated_message', index=3,
      number=4444, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=42,
  serialized_end=221,
)

_TESTMESSAGE.fields_by_name['optional_message'].message_type = _TESTMESSAGE
_TESTMESSAGE.fields_by_name['repeated_message'].message_type = _TESTMESSAGE
DESCRIPTOR.message_types_by_name['TestMessage'] = _TESTMESSAGE

class TestMessage(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TESTMESSAGE

  # @@protoc_insertion_point(class_scope:pomagma.protobuf.TestMessage)


# @@protoc_insertion_point(module_scope)
