# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: language.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='language.proto',
  package='pomagma.protobuf',
  serialized_pb='\n\x0elanguage.proto\x12\x10pomagma.protobuf\"\xa1\x01\n\x0cWeightedTerm\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x33\n\x05\x61rity\x18\x02 \x02(\x0e\x32$.pomagma.protobuf.WeightedTerm.Arity\x12\x0e\n\x06weight\x18\x03 \x02(\x02\">\n\x05\x41rity\x12\x0b\n\x07NULLARY\x10\x01\x12\r\n\tINJECTIVE\x10\x02\x12\n\n\x06\x42INARY\x10\x03\x12\r\n\tSYMMETRIC\x10\x04\"9\n\x08Language\x12-\n\x05terms\x18\x01 \x03(\x0b\x32\x1e.pomagma.protobuf.WeightedTerm')



_WEIGHTEDTERM_ARITY = _descriptor.EnumDescriptor(
  name='Arity',
  full_name='pomagma.protobuf.WeightedTerm.Arity',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NULLARY', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INJECTIVE', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BINARY', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SYMMETRIC', index=3, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=136,
  serialized_end=198,
)


_WEIGHTEDTERM = _descriptor.Descriptor(
  name='WeightedTerm',
  full_name='pomagma.protobuf.WeightedTerm',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.protobuf.WeightedTerm.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='arity', full_name='pomagma.protobuf.WeightedTerm.arity', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='weight', full_name='pomagma.protobuf.WeightedTerm.weight', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _WEIGHTEDTERM_ARITY,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=37,
  serialized_end=198,
)


_LANGUAGE = _descriptor.Descriptor(
  name='Language',
  full_name='pomagma.protobuf.Language',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='terms', full_name='pomagma.protobuf.Language.terms', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=200,
  serialized_end=257,
)

_WEIGHTEDTERM.fields_by_name['arity'].enum_type = _WEIGHTEDTERM_ARITY
_WEIGHTEDTERM_ARITY.containing_type = _WEIGHTEDTERM;
_LANGUAGE.fields_by_name['terms'].message_type = _WEIGHTEDTERM
DESCRIPTOR.message_types_by_name['WeightedTerm'] = _WEIGHTEDTERM
DESCRIPTOR.message_types_by_name['Language'] = _LANGUAGE

class WeightedTerm(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _WEIGHTEDTERM

  # @@protoc_insertion_point(class_scope:pomagma.protobuf.WeightedTerm)

class Language(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _LANGUAGE

  # @@protoc_insertion_point(class_scope:pomagma.protobuf.Language)


# @@protoc_insertion_point(module_scope)
