# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: syntax.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='syntax.proto',
  package='pomagma.compiler.protobuf',
  serialized_pb='\n\x0csyntax.proto\x12\x19pomagma.compiler.protobuf\"\xad\x01\n\tSignature\x12\x19\n\x11nullary_functions\x18\x04 \x03(\t\x12\x1b\n\x13injective_functions\x18\x06 \x03(\t\x12\x18\n\x10\x62inary_functions\x18\x07 \x03(\t\x12\x1b\n\x13symmetric_functions\x18\x08 \x03(\t\x12\x17\n\x0funary_relations\x18\t \x03(\t\x12\x18\n\x10\x62inary_relations\x18\n \x03(\t\"\x1c\n\nExpression\x12\x0e\n\x06polish\x18\x01 \x02(\t\"\x80\x01\n\x07Sequent\x12:\n\x0b\x61ntecedents\x18\x01 \x03(\x0b\x32%.pomagma.compiler.protobuf.Expression\x12\x39\n\nsuccedents\x18\x02 \x03(\x0b\x32%.pomagma.compiler.protobuf.Expression\"q\n\x06Theory\x12\x34\n\x05\x66\x61\x63ts\x18\x01 \x03(\x0b\x32%.pomagma.compiler.protobuf.Expression\x12\x31\n\x05rules\x18\x02 \x03(\x0b\x32\".pomagma.compiler.protobuf.Sequent\"z\n\x0cPresentation\x12\x37\n\tsignature\x18\x01 \x01(\x0b\x32$.pomagma.compiler.protobuf.Signature\x12\x31\n\x06theory\x18\x02 \x01(\x0b\x32!.pomagma.compiler.protobuf.Theory\"\x9d\x01\n\x06\x43orpus\x12\x41\n\x0b\x64\x65\x66initions\x18\x01 \x03(\x0b\x32,.pomagma.compiler.protobuf.Corpus.Definition\x1aP\n\nDefinition\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x34\n\x05value\x18\x02 \x02(\x0b\x32%.pomagma.compiler.protobuf.Expression')




_SIGNATURE = _descriptor.Descriptor(
  name='Signature',
  full_name='pomagma.compiler.protobuf.Signature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nullary_functions', full_name='pomagma.compiler.protobuf.Signature.nullary_functions', index=0,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='injective_functions', full_name='pomagma.compiler.protobuf.Signature.injective_functions', index=1,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='binary_functions', full_name='pomagma.compiler.protobuf.Signature.binary_functions', index=2,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symmetric_functions', full_name='pomagma.compiler.protobuf.Signature.symmetric_functions', index=3,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='unary_relations', full_name='pomagma.compiler.protobuf.Signature.unary_relations', index=4,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='binary_relations', full_name='pomagma.compiler.protobuf.Signature.binary_relations', index=5,
      number=10, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=44,
  serialized_end=217,
)


_EXPRESSION = _descriptor.Descriptor(
  name='Expression',
  full_name='pomagma.compiler.protobuf.Expression',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='polish', full_name='pomagma.compiler.protobuf.Expression.polish', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=219,
  serialized_end=247,
)


_SEQUENT = _descriptor.Descriptor(
  name='Sequent',
  full_name='pomagma.compiler.protobuf.Sequent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='antecedents', full_name='pomagma.compiler.protobuf.Sequent.antecedents', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='succedents', full_name='pomagma.compiler.protobuf.Sequent.succedents', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=250,
  serialized_end=378,
)


_THEORY = _descriptor.Descriptor(
  name='Theory',
  full_name='pomagma.compiler.protobuf.Theory',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='facts', full_name='pomagma.compiler.protobuf.Theory.facts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rules', full_name='pomagma.compiler.protobuf.Theory.rules', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=380,
  serialized_end=493,
)


_PRESENTATION = _descriptor.Descriptor(
  name='Presentation',
  full_name='pomagma.compiler.protobuf.Presentation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='signature', full_name='pomagma.compiler.protobuf.Presentation.signature', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='theory', full_name='pomagma.compiler.protobuf.Presentation.theory', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=495,
  serialized_end=617,
)


_CORPUS_DEFINITION = _descriptor.Descriptor(
  name='Definition',
  full_name='pomagma.compiler.protobuf.Corpus.Definition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.compiler.protobuf.Corpus.Definition.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='pomagma.compiler.protobuf.Corpus.Definition.value', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=697,
  serialized_end=777,
)

_CORPUS = _descriptor.Descriptor(
  name='Corpus',
  full_name='pomagma.compiler.protobuf.Corpus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='definitions', full_name='pomagma.compiler.protobuf.Corpus.definitions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CORPUS_DEFINITION, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=620,
  serialized_end=777,
)

_SEQUENT.fields_by_name['antecedents'].message_type = _EXPRESSION
_SEQUENT.fields_by_name['succedents'].message_type = _EXPRESSION
_THEORY.fields_by_name['facts'].message_type = _EXPRESSION
_THEORY.fields_by_name['rules'].message_type = _SEQUENT
_PRESENTATION.fields_by_name['signature'].message_type = _SIGNATURE
_PRESENTATION.fields_by_name['theory'].message_type = _THEORY
_CORPUS_DEFINITION.fields_by_name['value'].message_type = _EXPRESSION
_CORPUS_DEFINITION.containing_type = _CORPUS;
_CORPUS.fields_by_name['definitions'].message_type = _CORPUS_DEFINITION
DESCRIPTOR.message_types_by_name['Signature'] = _SIGNATURE
DESCRIPTOR.message_types_by_name['Expression'] = _EXPRESSION
DESCRIPTOR.message_types_by_name['Sequent'] = _SEQUENT
DESCRIPTOR.message_types_by_name['Theory'] = _THEORY
DESCRIPTOR.message_types_by_name['Presentation'] = _PRESENTATION
DESCRIPTOR.message_types_by_name['Corpus'] = _CORPUS

class Signature(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SIGNATURE

  # @@protoc_insertion_point(class_scope:pomagma.compiler.protobuf.Signature)

class Expression(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _EXPRESSION

  # @@protoc_insertion_point(class_scope:pomagma.compiler.protobuf.Expression)

class Sequent(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SEQUENT

  # @@protoc_insertion_point(class_scope:pomagma.compiler.protobuf.Sequent)

class Theory(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _THEORY

  # @@protoc_insertion_point(class_scope:pomagma.compiler.protobuf.Theory)

class Presentation(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PRESENTATION

  # @@protoc_insertion_point(class_scope:pomagma.compiler.protobuf.Presentation)

class Corpus(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Definition(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _CORPUS_DEFINITION

    # @@protoc_insertion_point(class_scope:pomagma.compiler.protobuf.Corpus.Definition)
  DESCRIPTOR = _CORPUS

  # @@protoc_insertion_point(class_scope:pomagma.compiler.protobuf.Corpus)


# @@protoc_insertion_point(module_scope)
