# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: structure.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='structure.proto',
  package='pomagma.atlas.protobuf',
  serialized_pb='\n\x0fstructure.proto\x12\x16pomagma.atlas.protobuf\"_\n\x05ObMap\x12\x0f\n\x03key\x18\x01 \x03(\rB\x02\x10\x01\x12\x0f\n\x03val\x18\x02 \x03(\rB\x02\x10\x01\x12\x1e\n\x12key_diff_minus_one\x18\x03 \x03(\rB\x02\x10\x01\x12\x14\n\x08val_diff\x18\x04 \x03(\x05\x42\x02\x10\x01\"\x16\n\x05ObSet\x12\r\n\x05\x64\x65nse\x18\x01 \x02(\x0c\"9\n\x07\x43\x61rrier\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x12\n\nitem_count\x18\x03 \x01(\r\"I\n\x0fNullaryFunction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x0b\n\x03val\x18\x03 \x01(\r\x12\r\n\x05\x62lobs\x18\x0f \x03(\x0c\"f\n\rUnaryFunction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12*\n\x03map\x18\x03 \x01(\x0b\x32\x1d.pomagma.atlas.protobuf.ObMap\x12\r\n\x05\x62lobs\x18\x0f \x03(\x0c\"\xb9\x01\n\x0e\x42inaryFunction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x38\n\x04rows\x18\x03 \x03(\x0b\x32*.pomagma.atlas.protobuf.BinaryFunction.Row\x12\r\n\x05\x62lobs\x18\x0f \x03(\x0c\x1a\x42\n\x03Row\x12\x0b\n\x03lhs\x18\x01 \x02(\r\x12.\n\x07rhs_val\x18\x02 \x02(\x0b\x32\x1d.pomagma.atlas.protobuf.ObMap\"f\n\rUnaryRelation\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12*\n\x03set\x18\x03 \x01(\x0b\x32\x1d.pomagma.atlas.protobuf.ObSet\x12\r\n\x05\x62lobs\x18\x0f \x03(\x0c\"\xb5\x01\n\x0e\x42inaryRelation\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x38\n\x04rows\x18\x03 \x03(\x0b\x32*.pomagma.atlas.protobuf.BinaryRelation.Row\x12\r\n\x05\x62lobs\x18\x0f \x03(\x0c\x1a>\n\x03Row\x12\x0b\n\x03lhs\x18\x01 \x02(\r\x12*\n\x03rhs\x18\x02 \x02(\x0b\x32\x1d.pomagma.atlas.protobuf.ObSet\"\xea\x03\n\tStructure\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x30\n\x07\x63\x61rrier\x18\x03 \x01(\x0b\x32\x1f.pomagma.atlas.protobuf.Carrier\x12\x42\n\x11nullary_functions\x18\x04 \x03(\x0b\x32\'.pomagma.atlas.protobuf.NullaryFunction\x12\x42\n\x13injective_functions\x18\x06 \x03(\x0b\x32%.pomagma.atlas.protobuf.UnaryFunction\x12@\n\x10\x62inary_functions\x18\x07 \x03(\x0b\x32&.pomagma.atlas.protobuf.BinaryFunction\x12\x43\n\x13symmetric_functions\x18\x08 \x03(\x0b\x32&.pomagma.atlas.protobuf.BinaryFunction\x12>\n\x0funary_relations\x18\t \x03(\x0b\x32%.pomagma.atlas.protobuf.UnaryRelation\x12@\n\x10\x62inary_relations\x18\n \x03(\x0b\x32&.pomagma.atlas.protobuf.BinaryRelation\"\x7f\n\x05\x43hart\x12\x1d\n\x15min_presentation_blob\x18\x01 \x02(\x0c\x12\x1d\n\x15max_presentation_blob\x18\x02 \x02(\x0c\x12\x16\n\x0estructure_blob\x18\x03 \x02(\x0c\x12 \n\x18subsumed_structure_blobs\x18\x04 \x03(\x0c\"6\n\x05\x41tlas\x12-\n\x06\x63harts\x18\x01 \x03(\x0b\x32\x1d.pomagma.atlas.protobuf.Chart')




_OBMAP = _descriptor.Descriptor(
  name='ObMap',
  full_name='pomagma.atlas.protobuf.ObMap',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='pomagma.atlas.protobuf.ObMap.key', index=0,
      number=1, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')),
    _descriptor.FieldDescriptor(
      name='val', full_name='pomagma.atlas.protobuf.ObMap.val', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')),
    _descriptor.FieldDescriptor(
      name='key_diff_minus_one', full_name='pomagma.atlas.protobuf.ObMap.key_diff_minus_one', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')),
    _descriptor.FieldDescriptor(
      name='val_diff', full_name='pomagma.atlas.protobuf.ObMap.val_diff', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=43,
  serialized_end=138,
)


_OBSET = _descriptor.Descriptor(
  name='ObSet',
  full_name='pomagma.atlas.protobuf.ObSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dense', full_name='pomagma.atlas.protobuf.ObSet.dense', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=140,
  serialized_end=162,
)


_CARRIER = _descriptor.Descriptor(
  name='Carrier',
  full_name='pomagma.atlas.protobuf.Carrier',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.atlas.protobuf.Carrier.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='pomagma.atlas.protobuf.Carrier.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='item_count', full_name='pomagma.atlas.protobuf.Carrier.item_count', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=164,
  serialized_end=221,
)


_NULLARYFUNCTION = _descriptor.Descriptor(
  name='NullaryFunction',
  full_name='pomagma.atlas.protobuf.NullaryFunction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.atlas.protobuf.NullaryFunction.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='pomagma.atlas.protobuf.NullaryFunction.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='val', full_name='pomagma.atlas.protobuf.NullaryFunction.val', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='pomagma.atlas.protobuf.NullaryFunction.blobs', index=3,
      number=15, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=223,
  serialized_end=296,
)


_UNARYFUNCTION = _descriptor.Descriptor(
  name='UnaryFunction',
  full_name='pomagma.atlas.protobuf.UnaryFunction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.atlas.protobuf.UnaryFunction.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='pomagma.atlas.protobuf.UnaryFunction.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map', full_name='pomagma.atlas.protobuf.UnaryFunction.map', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='pomagma.atlas.protobuf.UnaryFunction.blobs', index=3,
      number=15, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=298,
  serialized_end=400,
)


_BINARYFUNCTION_ROW = _descriptor.Descriptor(
  name='Row',
  full_name='pomagma.atlas.protobuf.BinaryFunction.Row',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lhs', full_name='pomagma.atlas.protobuf.BinaryFunction.Row.lhs', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rhs_val', full_name='pomagma.atlas.protobuf.BinaryFunction.Row.rhs_val', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=522,
  serialized_end=588,
)

_BINARYFUNCTION = _descriptor.Descriptor(
  name='BinaryFunction',
  full_name='pomagma.atlas.protobuf.BinaryFunction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.atlas.protobuf.BinaryFunction.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='pomagma.atlas.protobuf.BinaryFunction.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rows', full_name='pomagma.atlas.protobuf.BinaryFunction.rows', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='pomagma.atlas.protobuf.BinaryFunction.blobs', index=3,
      number=15, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_BINARYFUNCTION_ROW, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=403,
  serialized_end=588,
)


_UNARYRELATION = _descriptor.Descriptor(
  name='UnaryRelation',
  full_name='pomagma.atlas.protobuf.UnaryRelation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.atlas.protobuf.UnaryRelation.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='pomagma.atlas.protobuf.UnaryRelation.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='set', full_name='pomagma.atlas.protobuf.UnaryRelation.set', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='pomagma.atlas.protobuf.UnaryRelation.blobs', index=3,
      number=15, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=590,
  serialized_end=692,
)


_BINARYRELATION_ROW = _descriptor.Descriptor(
  name='Row',
  full_name='pomagma.atlas.protobuf.BinaryRelation.Row',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lhs', full_name='pomagma.atlas.protobuf.BinaryRelation.Row.lhs', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rhs', full_name='pomagma.atlas.protobuf.BinaryRelation.Row.rhs', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=814,
  serialized_end=876,
)

_BINARYRELATION = _descriptor.Descriptor(
  name='BinaryRelation',
  full_name='pomagma.atlas.protobuf.BinaryRelation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.atlas.protobuf.BinaryRelation.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='pomagma.atlas.protobuf.BinaryRelation.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rows', full_name='pomagma.atlas.protobuf.BinaryRelation.rows', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='pomagma.atlas.protobuf.BinaryRelation.blobs', index=3,
      number=15, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_BINARYRELATION_ROW, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=695,
  serialized_end=876,
)


_STRUCTURE = _descriptor.Descriptor(
  name='Structure',
  full_name='pomagma.atlas.protobuf.Structure',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.atlas.protobuf.Structure.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='pomagma.atlas.protobuf.Structure.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='carrier', full_name='pomagma.atlas.protobuf.Structure.carrier', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='nullary_functions', full_name='pomagma.atlas.protobuf.Structure.nullary_functions', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='injective_functions', full_name='pomagma.atlas.protobuf.Structure.injective_functions', index=4,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='binary_functions', full_name='pomagma.atlas.protobuf.Structure.binary_functions', index=5,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symmetric_functions', full_name='pomagma.atlas.protobuf.Structure.symmetric_functions', index=6,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='unary_relations', full_name='pomagma.atlas.protobuf.Structure.unary_relations', index=7,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='binary_relations', full_name='pomagma.atlas.protobuf.Structure.binary_relations', index=8,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=879,
  serialized_end=1369,
)


_CHART = _descriptor.Descriptor(
  name='Chart',
  full_name='pomagma.atlas.protobuf.Chart',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='min_presentation_blob', full_name='pomagma.atlas.protobuf.Chart.min_presentation_blob', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_presentation_blob', full_name='pomagma.atlas.protobuf.Chart.max_presentation_blob', index=1,
      number=2, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='structure_blob', full_name='pomagma.atlas.protobuf.Chart.structure_blob', index=2,
      number=3, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='subsumed_structure_blobs', full_name='pomagma.atlas.protobuf.Chart.subsumed_structure_blobs', index=3,
      number=4, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1371,
  serialized_end=1498,
)


_ATLAS = _descriptor.Descriptor(
  name='Atlas',
  full_name='pomagma.atlas.protobuf.Atlas',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='charts', full_name='pomagma.atlas.protobuf.Atlas.charts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1500,
  serialized_end=1554,
)

_UNARYFUNCTION.fields_by_name['map'].message_type = _OBMAP
_BINARYFUNCTION_ROW.fields_by_name['rhs_val'].message_type = _OBMAP
_BINARYFUNCTION_ROW.containing_type = _BINARYFUNCTION;
_BINARYFUNCTION.fields_by_name['rows'].message_type = _BINARYFUNCTION_ROW
_UNARYRELATION.fields_by_name['set'].message_type = _OBSET
_BINARYRELATION_ROW.fields_by_name['rhs'].message_type = _OBSET
_BINARYRELATION_ROW.containing_type = _BINARYRELATION;
_BINARYRELATION.fields_by_name['rows'].message_type = _BINARYRELATION_ROW
_STRUCTURE.fields_by_name['carrier'].message_type = _CARRIER
_STRUCTURE.fields_by_name['nullary_functions'].message_type = _NULLARYFUNCTION
_STRUCTURE.fields_by_name['injective_functions'].message_type = _UNARYFUNCTION
_STRUCTURE.fields_by_name['binary_functions'].message_type = _BINARYFUNCTION
_STRUCTURE.fields_by_name['symmetric_functions'].message_type = _BINARYFUNCTION
_STRUCTURE.fields_by_name['unary_relations'].message_type = _UNARYRELATION
_STRUCTURE.fields_by_name['binary_relations'].message_type = _BINARYRELATION
_ATLAS.fields_by_name['charts'].message_type = _CHART
DESCRIPTOR.message_types_by_name['ObMap'] = _OBMAP
DESCRIPTOR.message_types_by_name['ObSet'] = _OBSET
DESCRIPTOR.message_types_by_name['Carrier'] = _CARRIER
DESCRIPTOR.message_types_by_name['NullaryFunction'] = _NULLARYFUNCTION
DESCRIPTOR.message_types_by_name['UnaryFunction'] = _UNARYFUNCTION
DESCRIPTOR.message_types_by_name['BinaryFunction'] = _BINARYFUNCTION
DESCRIPTOR.message_types_by_name['UnaryRelation'] = _UNARYRELATION
DESCRIPTOR.message_types_by_name['BinaryRelation'] = _BINARYRELATION
DESCRIPTOR.message_types_by_name['Structure'] = _STRUCTURE
DESCRIPTOR.message_types_by_name['Chart'] = _CHART
DESCRIPTOR.message_types_by_name['Atlas'] = _ATLAS

class ObMap(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _OBMAP

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.ObMap)

class ObSet(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _OBSET

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.ObSet)

class Carrier(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CARRIER

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.Carrier)

class NullaryFunction(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _NULLARYFUNCTION

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.NullaryFunction)

class UnaryFunction(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UNARYFUNCTION

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.UnaryFunction)

class BinaryFunction(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Row(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _BINARYFUNCTION_ROW

    # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.BinaryFunction.Row)
  DESCRIPTOR = _BINARYFUNCTION

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.BinaryFunction)

class UnaryRelation(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UNARYRELATION

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.UnaryRelation)

class BinaryRelation(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Row(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _BINARYRELATION_ROW

    # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.BinaryRelation.Row)
  DESCRIPTOR = _BINARYRELATION

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.BinaryRelation)

class Structure(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _STRUCTURE

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.Structure)

class Chart(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CHART

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.Chart)

class Atlas(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _ATLAS

  # @@protoc_insertion_point(class_scope:pomagma.atlas.protobuf.Atlas)


_OBMAP.fields_by_name['key'].has_options = True
_OBMAP.fields_by_name['key']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')
_OBMAP.fields_by_name['val'].has_options = True
_OBMAP.fields_by_name['val']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')
_OBMAP.fields_by_name['key_diff_minus_one'].has_options = True
_OBMAP.fields_by_name['key_diff_minus_one']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')
_OBMAP.fields_by_name['val_diff'].has_options = True
_OBMAP.fields_by_name['val_diff']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')
# @@protoc_insertion_point(module_scope)
