# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='messages.proto',
  package='pomagma.protobuf',
  serialized_pb='\n\x0emessages.proto\x12\x10pomagma.protobuf\"m\n\tHistogram\x12/\n\x05terms\x18\x01 \x03(\x0b\x32 .pomagma.protobuf.Histogram.Term\x1a/\n\x04Term\x12\r\n\x05\x63ount\x18\x01 \x02(\x04\x12\n\n\x02ob\x18\x02 \x01(\r\x12\x0c\n\x04name\x18\x03 \x01(\t\"\xa6\x07\n\x0e\x41nalystRequest\x12\x11\n\terror_log\x18\x01 \x03(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x46\n\x0etest_inference\x18\x03 \x01(\x0b\x32..pomagma.protobuf.AnalystRequest.TestInference\x12;\n\x08simplify\x18\x04 \x01(\x0b\x32).pomagma.protobuf.AnalystRequest.Simplify\x12;\n\x08validate\x18\x05 \x01(\x0b\x32).pomagma.protobuf.AnalystRequest.Validate\x12H\n\x0fvalidate_corpus\x18\x06 \x01(\x0b\x32/.pomagma.protobuf.AnalystRequest.ValidateCorpus\x12\x44\n\rget_histogram\x18\x07 \x01(\x0b\x32-.pomagma.protobuf.AnalystRequest.GetHistogram\x12\x42\n\x0c\x66it_language\x18\x08 \x01(\x0b\x32,.pomagma.protobuf.AnalystRequest.FitLanguage\x12\x35\n\x05solve\x18\t \x01(\x0b\x32&.pomagma.protobuf.AnalystRequest.Solve\x12\x46\n\x0evalidate_facts\x18\n \x01(\x0b\x32..pomagma.protobuf.AnalystRequest.ValidateFacts\x1a\x0f\n\rTestInference\x1a\x19\n\x08Simplify\x12\r\n\x05\x63odes\x18\x01 \x03(\t\x1a\x19\n\x08Validate\x12\r\n\x05\x63odes\x18\x01 \x03(\t\x1ay\n\x0eValidateCorpus\x12\x43\n\x05lines\x18\x01 \x03(\x0b\x32\x34.pomagma.protobuf.AnalystRequest.ValidateCorpus.Line\x1a\"\n\x04Line\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x02(\t\x1a\x0e\n\x0cGetHistogram\x1a=\n\x0b\x46itLanguage\x12.\n\thistogram\x18\x01 \x01(\x0b\x32\x1b.pomagma.protobuf.Histogram\x1a/\n\x05Solve\x12\x0f\n\x07program\x18\x01 \x02(\t\x12\x15\n\rmax_solutions\x18\x02 \x01(\x04\x1a\x1e\n\rValidateFacts\x12\r\n\x05\x66\x61\x63ts\x18\x01 \x03(\t\"\xa7\n\n\x0f\x41nalystResponse\x12\x11\n\terror_log\x18\x01 \x03(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12G\n\x0etest_inference\x18\x03 \x01(\x0b\x32/.pomagma.protobuf.AnalystResponse.TestInference\x12<\n\x08simplify\x18\x04 \x01(\x0b\x32*.pomagma.protobuf.AnalystResponse.Simplify\x12<\n\x08validate\x18\x05 \x01(\x0b\x32*.pomagma.protobuf.AnalystResponse.Validate\x12I\n\x0fvalidate_corpus\x18\x06 \x01(\x0b\x32\x30.pomagma.protobuf.AnalystResponse.ValidateCorpus\x12\x45\n\rget_histogram\x18\x07 \x01(\x0b\x32..pomagma.protobuf.AnalystResponse.GetHistogram\x12\x43\n\x0c\x66it_language\x18\x08 \x01(\x0b\x32-.pomagma.protobuf.AnalystResponse.FitLanguage\x12\x36\n\x05solve\x18\t \x01(\x0b\x32\'.pomagma.protobuf.AnalystResponse.Solve\x12G\n\x0evalidate_facts\x18\n \x01(\x0b\x32/.pomagma.protobuf.AnalystResponse.ValidateFacts\x1a\xa2\x01\n\x08Validity\x12>\n\x06is_top\x18\x01 \x02(\x0e\x32\'.pomagma.protobuf.AnalystResponse.Trool:\x05MAYBE\x12>\n\x06is_bot\x18\x02 \x02(\x0e\x32\'.pomagma.protobuf.AnalystResponse.Trool:\x05MAYBE\x12\x16\n\x07pending\x18\x03 \x02(\x08:\x05\x66\x61lse\x1a#\n\rTestInference\x12\x12\n\nfail_count\x18\x01 \x02(\x04\x1a\x19\n\x08Simplify\x12\r\n\x05\x63odes\x18\x01 \x03(\t\x1aG\n\x08Validate\x12;\n\x07results\x18\x01 \x03(\x0b\x32*.pomagma.protobuf.AnalystResponse.Validity\x1aM\n\x0eValidateCorpus\x12;\n\x07results\x18\x01 \x03(\x0b\x32*.pomagma.protobuf.AnalystResponse.Validity\x1a>\n\x0cGetHistogram\x12.\n\thistogram\x18\x01 \x02(\x0b\x32\x1b.pomagma.protobuf.Histogram\x1az\n\x0b\x46itLanguage\x12\x45\n\x07symbols\x18\x01 \x03(\x0b\x32\x34.pomagma.protobuf.AnalystResponse.FitLanguage.Symbol\x1a$\n\x06Symbol\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0c\n\x04prob\x18\x02 \x02(\x02\x1a,\n\x05Solve\x12\x11\n\tnecessary\x18\x01 \x03(\t\x12\x10\n\x08possible\x18\x02 \x03(\t\x1aH\n\rValidateFacts\x12\x37\n\x06result\x18\x01 \x02(\x0e\x32\'.pomagma.protobuf.AnalystResponse.Trool\"\'\n\x05Trool\x12\t\n\x05MAYBE\x10\x00\x12\t\n\x05\x46\x41LSE\x10\x01\x12\x08\n\x04TRUE\x10\x02')



_ANALYSTRESPONSE_TROOL = _descriptor.EnumDescriptor(
  name='Trool',
  full_name='pomagma.protobuf.AnalystResponse.Trool',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MAYBE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FALSE', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRUE', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2365,
  serialized_end=2404,
)


_HISTOGRAM_TERM = _descriptor.Descriptor(
  name='Term',
  full_name='pomagma.protobuf.Histogram.Term',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='count', full_name='pomagma.protobuf.Histogram.Term.count', index=0,
      number=1, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ob', full_name='pomagma.protobuf.Histogram.Term.ob', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.protobuf.Histogram.Term.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=98,
  serialized_end=145,
)

_HISTOGRAM = _descriptor.Descriptor(
  name='Histogram',
  full_name='pomagma.protobuf.Histogram',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='terms', full_name='pomagma.protobuf.Histogram.terms', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_HISTOGRAM_TERM, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=36,
  serialized_end=145,
)


_ANALYSTREQUEST_TESTINFERENCE = _descriptor.Descriptor(
  name='TestInference',
  full_name='pomagma.protobuf.AnalystRequest.TestInference',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=730,
  serialized_end=745,
)

_ANALYSTREQUEST_SIMPLIFY = _descriptor.Descriptor(
  name='Simplify',
  full_name='pomagma.protobuf.AnalystRequest.Simplify',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='codes', full_name='pomagma.protobuf.AnalystRequest.Simplify.codes', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=747,
  serialized_end=772,
)

_ANALYSTREQUEST_VALIDATE = _descriptor.Descriptor(
  name='Validate',
  full_name='pomagma.protobuf.AnalystRequest.Validate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='codes', full_name='pomagma.protobuf.AnalystRequest.Validate.codes', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=774,
  serialized_end=799,
)

_ANALYSTREQUEST_VALIDATECORPUS_LINE = _descriptor.Descriptor(
  name='Line',
  full_name='pomagma.protobuf.AnalystRequest.ValidateCorpus.Line',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.protobuf.AnalystRequest.ValidateCorpus.Line.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='code', full_name='pomagma.protobuf.AnalystRequest.ValidateCorpus.Line.code', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=888,
  serialized_end=922,
)

_ANALYSTREQUEST_VALIDATECORPUS = _descriptor.Descriptor(
  name='ValidateCorpus',
  full_name='pomagma.protobuf.AnalystRequest.ValidateCorpus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lines', full_name='pomagma.protobuf.AnalystRequest.ValidateCorpus.lines', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_ANALYSTREQUEST_VALIDATECORPUS_LINE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=801,
  serialized_end=922,
)

_ANALYSTREQUEST_GETHISTOGRAM = _descriptor.Descriptor(
  name='GetHistogram',
  full_name='pomagma.protobuf.AnalystRequest.GetHistogram',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=924,
  serialized_end=938,
)

_ANALYSTREQUEST_FITLANGUAGE = _descriptor.Descriptor(
  name='FitLanguage',
  full_name='pomagma.protobuf.AnalystRequest.FitLanguage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='histogram', full_name='pomagma.protobuf.AnalystRequest.FitLanguage.histogram', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=940,
  serialized_end=1001,
)

_ANALYSTREQUEST_SOLVE = _descriptor.Descriptor(
  name='Solve',
  full_name='pomagma.protobuf.AnalystRequest.Solve',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='program', full_name='pomagma.protobuf.AnalystRequest.Solve.program', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_solutions', full_name='pomagma.protobuf.AnalystRequest.Solve.max_solutions', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1003,
  serialized_end=1050,
)

_ANALYSTREQUEST_VALIDATEFACTS = _descriptor.Descriptor(
  name='ValidateFacts',
  full_name='pomagma.protobuf.AnalystRequest.ValidateFacts',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='facts', full_name='pomagma.protobuf.AnalystRequest.ValidateFacts.facts', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1052,
  serialized_end=1082,
)

_ANALYSTREQUEST = _descriptor.Descriptor(
  name='AnalystRequest',
  full_name='pomagma.protobuf.AnalystRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_log', full_name='pomagma.protobuf.AnalystRequest.error_log', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='pomagma.protobuf.AnalystRequest.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='test_inference', full_name='pomagma.protobuf.AnalystRequest.test_inference', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='simplify', full_name='pomagma.protobuf.AnalystRequest.simplify', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='validate', full_name='pomagma.protobuf.AnalystRequest.validate', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='validate_corpus', full_name='pomagma.protobuf.AnalystRequest.validate_corpus', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='get_histogram', full_name='pomagma.protobuf.AnalystRequest.get_histogram', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='fit_language', full_name='pomagma.protobuf.AnalystRequest.fit_language', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='solve', full_name='pomagma.protobuf.AnalystRequest.solve', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='validate_facts', full_name='pomagma.protobuf.AnalystRequest.validate_facts', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_ANALYSTREQUEST_TESTINFERENCE, _ANALYSTREQUEST_SIMPLIFY, _ANALYSTREQUEST_VALIDATE, _ANALYSTREQUEST_VALIDATECORPUS, _ANALYSTREQUEST_GETHISTOGRAM, _ANALYSTREQUEST_FITLANGUAGE, _ANALYSTREQUEST_SOLVE, _ANALYSTREQUEST_VALIDATEFACTS, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=148,
  serialized_end=1082,
)


_ANALYSTRESPONSE_VALIDITY = _descriptor.Descriptor(
  name='Validity',
  full_name='pomagma.protobuf.AnalystResponse.Validity',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='is_top', full_name='pomagma.protobuf.AnalystResponse.Validity.is_top', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_bot', full_name='pomagma.protobuf.AnalystResponse.Validity.is_bot', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pending', full_name='pomagma.protobuf.AnalystResponse.Validity.pending', index=2,
      number=3, type=8, cpp_type=7, label=2,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1677,
  serialized_end=1839,
)

_ANALYSTRESPONSE_TESTINFERENCE = _descriptor.Descriptor(
  name='TestInference',
  full_name='pomagma.protobuf.AnalystResponse.TestInference',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fail_count', full_name='pomagma.protobuf.AnalystResponse.TestInference.fail_count', index=0,
      number=1, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1841,
  serialized_end=1876,
)

_ANALYSTRESPONSE_SIMPLIFY = _descriptor.Descriptor(
  name='Simplify',
  full_name='pomagma.protobuf.AnalystResponse.Simplify',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='codes', full_name='pomagma.protobuf.AnalystResponse.Simplify.codes', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=747,
  serialized_end=772,
)

_ANALYSTRESPONSE_VALIDATE = _descriptor.Descriptor(
  name='Validate',
  full_name='pomagma.protobuf.AnalystResponse.Validate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='results', full_name='pomagma.protobuf.AnalystResponse.Validate.results', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1905,
  serialized_end=1976,
)

_ANALYSTRESPONSE_VALIDATECORPUS = _descriptor.Descriptor(
  name='ValidateCorpus',
  full_name='pomagma.protobuf.AnalystResponse.ValidateCorpus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='results', full_name='pomagma.protobuf.AnalystResponse.ValidateCorpus.results', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1978,
  serialized_end=2055,
)

_ANALYSTRESPONSE_GETHISTOGRAM = _descriptor.Descriptor(
  name='GetHistogram',
  full_name='pomagma.protobuf.AnalystResponse.GetHistogram',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='histogram', full_name='pomagma.protobuf.AnalystResponse.GetHistogram.histogram', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2057,
  serialized_end=2119,
)

_ANALYSTRESPONSE_FITLANGUAGE_SYMBOL = _descriptor.Descriptor(
  name='Symbol',
  full_name='pomagma.protobuf.AnalystResponse.FitLanguage.Symbol',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pomagma.protobuf.AnalystResponse.FitLanguage.Symbol.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prob', full_name='pomagma.protobuf.AnalystResponse.FitLanguage.Symbol.prob', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2207,
  serialized_end=2243,
)

_ANALYSTRESPONSE_FITLANGUAGE = _descriptor.Descriptor(
  name='FitLanguage',
  full_name='pomagma.protobuf.AnalystResponse.FitLanguage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='symbols', full_name='pomagma.protobuf.AnalystResponse.FitLanguage.symbols', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_ANALYSTRESPONSE_FITLANGUAGE_SYMBOL, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2121,
  serialized_end=2243,
)

_ANALYSTRESPONSE_SOLVE = _descriptor.Descriptor(
  name='Solve',
  full_name='pomagma.protobuf.AnalystResponse.Solve',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='necessary', full_name='pomagma.protobuf.AnalystResponse.Solve.necessary', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='possible', full_name='pomagma.protobuf.AnalystResponse.Solve.possible', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2245,
  serialized_end=2289,
)

_ANALYSTRESPONSE_VALIDATEFACTS = _descriptor.Descriptor(
  name='ValidateFacts',
  full_name='pomagma.protobuf.AnalystResponse.ValidateFacts',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='pomagma.protobuf.AnalystResponse.ValidateFacts.result', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2291,
  serialized_end=2363,
)

_ANALYSTRESPONSE = _descriptor.Descriptor(
  name='AnalystResponse',
  full_name='pomagma.protobuf.AnalystResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_log', full_name='pomagma.protobuf.AnalystResponse.error_log', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='pomagma.protobuf.AnalystResponse.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='test_inference', full_name='pomagma.protobuf.AnalystResponse.test_inference', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='simplify', full_name='pomagma.protobuf.AnalystResponse.simplify', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='validate', full_name='pomagma.protobuf.AnalystResponse.validate', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='validate_corpus', full_name='pomagma.protobuf.AnalystResponse.validate_corpus', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='get_histogram', full_name='pomagma.protobuf.AnalystResponse.get_histogram', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='fit_language', full_name='pomagma.protobuf.AnalystResponse.fit_language', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='solve', full_name='pomagma.protobuf.AnalystResponse.solve', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='validate_facts', full_name='pomagma.protobuf.AnalystResponse.validate_facts', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_ANALYSTRESPONSE_VALIDITY, _ANALYSTRESPONSE_TESTINFERENCE, _ANALYSTRESPONSE_SIMPLIFY, _ANALYSTRESPONSE_VALIDATE, _ANALYSTRESPONSE_VALIDATECORPUS, _ANALYSTRESPONSE_GETHISTOGRAM, _ANALYSTRESPONSE_FITLANGUAGE, _ANALYSTRESPONSE_SOLVE, _ANALYSTRESPONSE_VALIDATEFACTS, ],
  enum_types=[
    _ANALYSTRESPONSE_TROOL,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1085,
  serialized_end=2404,
)

_HISTOGRAM_TERM.containing_type = _HISTOGRAM;
_HISTOGRAM.fields_by_name['terms'].message_type = _HISTOGRAM_TERM
_ANALYSTREQUEST_TESTINFERENCE.containing_type = _ANALYSTREQUEST;
_ANALYSTREQUEST_SIMPLIFY.containing_type = _ANALYSTREQUEST;
_ANALYSTREQUEST_VALIDATE.containing_type = _ANALYSTREQUEST;
_ANALYSTREQUEST_VALIDATECORPUS_LINE.containing_type = _ANALYSTREQUEST_VALIDATECORPUS;
_ANALYSTREQUEST_VALIDATECORPUS.fields_by_name['lines'].message_type = _ANALYSTREQUEST_VALIDATECORPUS_LINE
_ANALYSTREQUEST_VALIDATECORPUS.containing_type = _ANALYSTREQUEST;
_ANALYSTREQUEST_GETHISTOGRAM.containing_type = _ANALYSTREQUEST;
_ANALYSTREQUEST_FITLANGUAGE.fields_by_name['histogram'].message_type = _HISTOGRAM
_ANALYSTREQUEST_FITLANGUAGE.containing_type = _ANALYSTREQUEST;
_ANALYSTREQUEST_SOLVE.containing_type = _ANALYSTREQUEST;
_ANALYSTREQUEST_VALIDATEFACTS.containing_type = _ANALYSTREQUEST;
_ANALYSTREQUEST.fields_by_name['test_inference'].message_type = _ANALYSTREQUEST_TESTINFERENCE
_ANALYSTREQUEST.fields_by_name['simplify'].message_type = _ANALYSTREQUEST_SIMPLIFY
_ANALYSTREQUEST.fields_by_name['validate'].message_type = _ANALYSTREQUEST_VALIDATE
_ANALYSTREQUEST.fields_by_name['validate_corpus'].message_type = _ANALYSTREQUEST_VALIDATECORPUS
_ANALYSTREQUEST.fields_by_name['get_histogram'].message_type = _ANALYSTREQUEST_GETHISTOGRAM
_ANALYSTREQUEST.fields_by_name['fit_language'].message_type = _ANALYSTREQUEST_FITLANGUAGE
_ANALYSTREQUEST.fields_by_name['solve'].message_type = _ANALYSTREQUEST_SOLVE
_ANALYSTREQUEST.fields_by_name['validate_facts'].message_type = _ANALYSTREQUEST_VALIDATEFACTS
_ANALYSTRESPONSE_VALIDITY.fields_by_name['is_top'].enum_type = _ANALYSTRESPONSE_TROOL
_ANALYSTRESPONSE_VALIDITY.fields_by_name['is_bot'].enum_type = _ANALYSTRESPONSE_TROOL
_ANALYSTRESPONSE_VALIDITY.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE_TESTINFERENCE.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE_SIMPLIFY.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE_VALIDATE.fields_by_name['results'].message_type = _ANALYSTRESPONSE_VALIDITY
_ANALYSTRESPONSE_VALIDATE.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE_VALIDATECORPUS.fields_by_name['results'].message_type = _ANALYSTRESPONSE_VALIDITY
_ANALYSTRESPONSE_VALIDATECORPUS.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE_GETHISTOGRAM.fields_by_name['histogram'].message_type = _HISTOGRAM
_ANALYSTRESPONSE_GETHISTOGRAM.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE_FITLANGUAGE_SYMBOL.containing_type = _ANALYSTRESPONSE_FITLANGUAGE;
_ANALYSTRESPONSE_FITLANGUAGE.fields_by_name['symbols'].message_type = _ANALYSTRESPONSE_FITLANGUAGE_SYMBOL
_ANALYSTRESPONSE_FITLANGUAGE.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE_SOLVE.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE_VALIDATEFACTS.fields_by_name['result'].enum_type = _ANALYSTRESPONSE_TROOL
_ANALYSTRESPONSE_VALIDATEFACTS.containing_type = _ANALYSTRESPONSE;
_ANALYSTRESPONSE.fields_by_name['test_inference'].message_type = _ANALYSTRESPONSE_TESTINFERENCE
_ANALYSTRESPONSE.fields_by_name['simplify'].message_type = _ANALYSTRESPONSE_SIMPLIFY
_ANALYSTRESPONSE.fields_by_name['validate'].message_type = _ANALYSTRESPONSE_VALIDATE
_ANALYSTRESPONSE.fields_by_name['validate_corpus'].message_type = _ANALYSTRESPONSE_VALIDATECORPUS
_ANALYSTRESPONSE.fields_by_name['get_histogram'].message_type = _ANALYSTRESPONSE_GETHISTOGRAM
_ANALYSTRESPONSE.fields_by_name['fit_language'].message_type = _ANALYSTRESPONSE_FITLANGUAGE
_ANALYSTRESPONSE.fields_by_name['solve'].message_type = _ANALYSTRESPONSE_SOLVE
_ANALYSTRESPONSE.fields_by_name['validate_facts'].message_type = _ANALYSTRESPONSE_VALIDATEFACTS
_ANALYSTRESPONSE_TROOL.containing_type = _ANALYSTRESPONSE;
DESCRIPTOR.message_types_by_name['Histogram'] = _HISTOGRAM
DESCRIPTOR.message_types_by_name['AnalystRequest'] = _ANALYSTREQUEST
DESCRIPTOR.message_types_by_name['AnalystResponse'] = _ANALYSTRESPONSE

class Histogram(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Term(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _HISTOGRAM_TERM

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.Histogram.Term)
  DESCRIPTOR = _HISTOGRAM

  # @@protoc_insertion_point(class_scope:pomagma.protobuf.Histogram)

class AnalystRequest(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class TestInference(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTREQUEST_TESTINFERENCE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.TestInference)

  class Simplify(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTREQUEST_SIMPLIFY

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.Simplify)

  class Validate(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTREQUEST_VALIDATE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.Validate)

  class ValidateCorpus(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType

    class Line(_message.Message):
      __metaclass__ = _reflection.GeneratedProtocolMessageType
      DESCRIPTOR = _ANALYSTREQUEST_VALIDATECORPUS_LINE

      # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.ValidateCorpus.Line)
    DESCRIPTOR = _ANALYSTREQUEST_VALIDATECORPUS

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.ValidateCorpus)

  class GetHistogram(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTREQUEST_GETHISTOGRAM

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.GetHistogram)

  class FitLanguage(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTREQUEST_FITLANGUAGE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.FitLanguage)

  class Solve(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTREQUEST_SOLVE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.Solve)

  class ValidateFacts(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTREQUEST_VALIDATEFACTS

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest.ValidateFacts)
  DESCRIPTOR = _ANALYSTREQUEST

  # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystRequest)

class AnalystResponse(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Validity(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTRESPONSE_VALIDITY

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.Validity)

  class TestInference(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTRESPONSE_TESTINFERENCE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.TestInference)

  class Simplify(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTRESPONSE_SIMPLIFY

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.Simplify)

  class Validate(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTRESPONSE_VALIDATE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.Validate)

  class ValidateCorpus(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTRESPONSE_VALIDATECORPUS

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.ValidateCorpus)

  class GetHistogram(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTRESPONSE_GETHISTOGRAM

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.GetHistogram)

  class FitLanguage(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType

    class Symbol(_message.Message):
      __metaclass__ = _reflection.GeneratedProtocolMessageType
      DESCRIPTOR = _ANALYSTRESPONSE_FITLANGUAGE_SYMBOL

      # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.FitLanguage.Symbol)
    DESCRIPTOR = _ANALYSTRESPONSE_FITLANGUAGE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.FitLanguage)

  class Solve(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTRESPONSE_SOLVE

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.Solve)

  class ValidateFacts(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _ANALYSTRESPONSE_VALIDATEFACTS

    # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse.ValidateFacts)
  DESCRIPTOR = _ANALYSTRESPONSE

  # @@protoc_insertion_point(class_scope:pomagma.protobuf.AnalystResponse)


# @@protoc_insertion_point(module_scope)
